@@ -95,7 +95,7 @@ def do_lock(
                 ui.echo(format_resolution_impossible(err), err=True)
                 raise ResolutionImpossible("Unable to find a resolution") from None
             else:
-                data = format_lockfile(mapping, dependencies)
+                data = format_lockfile(project, mapping, dependencies)
                 spin.succeed(f"{termui.Emoji.LOCK} Lock successful")
             signals.post_lock.send(project, resolution=mapping, dry_run=dry_run)
 
@@ -184,7 +184,7 @@ def do_sync(
         dry_run,
         no_editable=no_editable,
         install_self=not no_self and "default" in groups and bool(project.meta.name),
-        use_install_cache=project.config["feature.install_cache"],
+        use_install_cache=project.config["install.cache"],
         reinstall=reinstall,
     )
     signals.pre_install.send(project, candidates=candidates, dry_run=dry_run)
@@ -224,7 +224,7 @@ def do_add(
         requirements[key] = r
     project.core.ui.echo(
         f"Adding packages to {group} {'dev-' if dev else ''}dependencies: "
-        + ", ".join(termui.green(key or "", bold=True) for key in requirements)
+        + ", ".join(termui.green(r.as_line(), bold=True) for r in requirements.values())
     )
     all_dependencies = project.all_dependencies
     group_deps = all_dependencies.setdefault(group, {})
@@ -240,8 +240,7 @@ def do_add(
     save_version_specifiers({group: deps_to_update}, resolved, save)
     if not dry_run:
         project.add_dependencies(deps_to_update, group, dev)
-    lockfile = project.lockfile
-    project.write_lockfile(lockfile, False)
+        project.write_lockfile(project.lockfile, False)
 
     if sync:
         do_sync(
@@ -468,6 +467,8 @@ def do_build(
         dest = project.root.joinpath(dest).as_posix()
     if clean:
         shutil.rmtree(dest, ignore_errors=True)
+    signals.pre_build.send(project, dest=dest, config_settings=config_settings)
+    artifacts: list[str] = []
     with project.core.ui.logging("build"):
         if sdist:
             project.core.ui.echo("Building sdist...")
@@ -475,12 +476,17 @@ def do_build(
                 dest, config_settings
             )
             project.core.ui.echo(f"Built sdist at {loc}")
+            artifacts.append(loc)
         if wheel:
             project.core.ui.echo("Building wheel...")
             loc = WheelBuilder(project.root, project.environment).build(
                 dest, config_settings
             )
             project.core.ui.echo(f"Built wheel at {loc}")
+            artifacts.append(loc)
+    signals.post_build.send(
+        project, artifacts=artifacts, config_settings=config_settings
+    )
 
 
 def do_init(
@@ -500,12 +506,13 @@ def do_init(
             "description": "",
             "authors": array_of_inline_tables([{"name": author, "email": email}]),
             "license": make_inline_table({"text": license}),
-            "urls": {"homepage": ""},
+            "urls": {"Homepage": ""},
             "dependencies": make_array([], True),
-            "requires-python": "" if python_requires == "*" else python_requires,
         },
         "build-system": {"requires": ["pdm-pep517"], "build-backend": "pdm.pep517.api"},
     }
+    if python_requires and python_requires != "*":
+        data["project"]["requires-python"] = python_requires  # type: ignore
     get_specifier(python_requires)
     if not project.pyproject:
         project._pyproject = data
@@ -741,58 +748,6 @@ def print_pep582_command(ui: termui.UI, shell: str = "AUTO") -> None:
     ui.echo(result)
 
 
-def migrate_pyproject(project: Project) -> None:
-    """Migrate the legacy pyproject format to PEP 621"""
-
-    if project.pyproject and "project" in project.pyproject:
-        pyproject = project.pyproject
-        settings = {}
-        updated_fields = []
-        for field in ("includes", "excludes", "build", "package-dir"):
-            if field in pyproject["project"]:
-                updated_fields.append(field)
-                settings[field] = pyproject["project"].pop(field)
-        if "dev-dependencies" in pyproject["project"]:
-            if pyproject["project"]["dev-dependencies"]:
-                settings["dev-dependencies"] = {
-                    "dev": pyproject["project"]["dev-dependencies"]
-                }
-            del pyproject["project"]["dev-dependencies"]
-            updated_fields.append("dev-dependencies")
-        if updated_fields:
-            if "tool" not in pyproject or "pdm" not in pyproject["tool"]:
-                pyproject.setdefault("tool", {})["pdm"] = tomlkit.table()
-            pyproject["tool"]["pdm"].update(settings)
-            project.pyproject = pyproject
-            project.write_pyproject()
-            project.core.ui.echo(
-                f"{termui.yellow('[AUTO-MIGRATION]')} These fields are moved from "
-                f"[project] to [tool.pdm] table: {updated_fields}",
-                err=True,
-            )
-        return
-
-    if not project.pyproject_file.exists() or not FORMATS["legacy"].check_fingerprint(
-        project, project.pyproject_file
-    ):
-        return
-
-    project.core.ui.echo(
-        f"{termui.yellow('[AUTO-MIGRATION]')} Legacy pdm 0.x metadata detected, "
-        "migrating to PEP 621...",
-        err=True,
-    )
-    do_import(project, str(project.pyproject_file), "legacy")
-    project.core.ui.echo(
-        termui.green("pyproject.toml")
-        + termui.yellow(
-            " has been migrated to PEP 621 successfully. "
-            "Now you can safely delete the legacy metadata under [tool.pdm] table."
-        ),
-        err=True,
-    )
-
-
 def get_latest_version(project: Project) -> str | None:
     """Get the latest version of PDM from PyPI, cache for 7 days"""
     from pdm.utils import get_finder
