@@ -2,6 +2,7 @@ from __future__ import annotations
 
 from typing import TYPE_CHECKING
 
+from packaging.specifiers import InvalidSpecifier, SpecifierSet
 from resolvelib import AbstractProvider
 
 from pdm.models.candidates import Candidate
@@ -37,7 +38,7 @@ class BaseProvider(AbstractProvider):
         self.overrides = overrides or {}
         self._known_depth: dict[str, int] = {}
 
-    def requirement_preference(self, requirement: Requirement) -> tuple:
+    def requirement_preference(self, requirement: Requirement) -> Comparable:
         """Return the preference of a requirement to find candidates.
 
         - Editable requirements are preferered.
@@ -45,14 +46,14 @@ class BaseProvider(AbstractProvider):
         - The one with narrower specifierset is preferred.
         """
         editable = requirement.editable
-        is_file = requirement.is_file_or_url
+        is_named = requirement.is_named
         is_prerelease = (
             requirement.prerelease
             or requirement.specifier is not None
             and bool(requirement.specifier.prereleases)
         )
         specifier_parts = len(requirement.specifier) if requirement.specifier else 0
-        return (editable, is_file, is_prerelease, specifier_parts)
+        return (not editable, is_named, not is_prerelease, -specifier_parts)
 
     def identify(self, requirement_or_candidate: Requirement | Candidate) -> str:
         return requirement_or_candidate.identify()
@@ -80,7 +81,8 @@ class BaseProvider(AbstractProvider):
                 for _, parent in information[identifier]
             )
             dep_depth = min(parent_depths) + 1
-        self._known_depth[identifier] = dep_depth
+        # Use the REAL identifier as it may be updated after candidate preparation.
+        self._known_depth[self.identify(next(candidates[identifier]))] = dep_depth
         is_file_or_url = any(
             not requirement.is_named for requirement, _ in information[identifier]
         )
@@ -107,11 +109,25 @@ class BaseProvider(AbstractProvider):
     def get_override_candidates(self, identifier: str) -> Iterable[Candidate]:
         requested = self.overrides[identifier]
         if is_url(requested):
-            requested = f"{identifier} @ {requested}"
+            req = f"{identifier} @ {requested}"
         else:
-            requested = f"{identifier}=={requested}"
-        req = parse_requirement(requested)
-        return self.repository.find_candidates(req, self.allow_prereleases)
+            try:
+                SpecifierSet(requested)
+            except InvalidSpecifier:  # handle bare versions
+                req = f"{identifier}=={requested}"
+            else:
+                req = f"{identifier}{requested}"
+        return self._find_candidates(parse_requirement(req))
+
+    def _find_candidates(self, requirement: Requirement) -> Iterable[Candidate]:
+        if not requirement.is_named:
+            can = Candidate(requirement)
+            can.prepare(self.repository.environment)
+            return [can]
+        else:
+            return self.repository.find_candidates(
+                requirement, requirement.prerelease or self.allow_prereleases
+            )
 
     def find_matches(
         self,
@@ -121,25 +137,12 @@ class BaseProvider(AbstractProvider):
     ) -> Iterable[Candidate]:
         incompat = list(incompatibilities[identifier])
         if identifier == "python":
-            candidates = find_python_matches(
-                identifier, requirements, self.repository.environment
-            )
+            candidates = find_python_matches(identifier, requirements)
             return [c for c in candidates if c not in incompat]
         elif identifier in self.overrides:
             return self.get_override_candidates(identifier)
-        reqs = sorted(
-            requirements[identifier], key=self.requirement_preference, reverse=True
-        )
-        file_req = next((req for req in reqs if not req.is_named), None)
-        if file_req:
-            can = Candidate(file_req, self.repository.environment)
-            can.metadata
-            candidates = [can]
-        else:
-            req = reqs[0]
-            candidates = self.repository.find_candidates(
-                req, req.prerelease or self.allow_prereleases
-            )
+        reqs = sorted(requirements[identifier], key=self.requirement_preference)
+        candidates = self._find_candidates(reqs[0])
         return [
             can
             for can in candidates
@@ -181,8 +184,7 @@ class BaseProvider(AbstractProvider):
                 continue
             dep.requires_python &= candidate.req.requires_python
             valid_deps.append(dep)
-        candidate_key = self.identify(candidate)
-        self.fetched_dependencies[candidate_key] = valid_deps[:]
+        self.fetched_dependencies[self.identify(candidate)] = valid_deps[:]
         # A candidate contributes to the Python requirements only when:
         # It isn't an optional dependency, or the requires-python doesn't cover
         # the req's requires-python.
